//
// This code is part of https://developer.mescius.com/document-solutions/dot-net-word-api/demos.
// Copyright (c) MESCIUS inc. All rights reserved.
//
using System;
using System.IO;
using System.Diagnostics;
using System.Reflection;
using GrapeCity.Documents.Word;
using GrapeCity.Documents.Word.Layout;
using GrapeCity.Documents.Text;
//
// This program creates an instance of the TateChuYoko class and uses it to generate a Word file.
// The file is also saved as PDF (using GcWordDocument.SaveAsPdf() extension method).
// The GrapeCity packages needed to build and run this sample are available on NuGet.org.
//
// Command line switches:
// /y, -y -- assume all defaults, overwrite existing files.
// /q, -q -- quiet operation.
// /n, -n -- generate repeatable 'random' data.
// /f path, -f path -- use 'name' as base file path/name.
//
var yes2all = false;
var quiet = false;
var norandom = false;
MemoryStream outMs = null;
TextWriter outTw = null;
try
{
    var fname = "TateChuYoko";
    for (int i = 0; i < args.Length; ++i)
    {
        switch (args[i].ToLowerInvariant())
        {
            case "/y":
            case "-y":
                yes2all = true;
                break;
            case "/q":
            case "-q":
                quiet = true;
                break;
            case "/n":
            case "-n":
                norandom = true;
                break;
            case "/f":
            case "-f":
                if (i + 1 < args.Length)
                {
                    if (Directory.Exists(args[++i]))
                        fname = Path.Combine(args[i], fname);
                    else
                        fname = args[i];
                }
                break;
        }
    }
    if (norandom)
        DsWordWeb.Demos.Util.NoRandom = true;
    if (!yes2all)
        quiet = false;
    if (quiet)
    {
        outMs = new MemoryStream();
        outTw = new StreamWriter(outMs);
        Console.SetOut(outTw);
    }

    var lk = Environment.GetEnvironmentVariable("DOCSOL_LKEY");
    if (!string.IsNullOrEmpty(lk))
        GcWordDocument.SetLicenseKey(lk);

    var docxName = Path.ChangeExtension(fname, ".docx");
    var pdfName = Path.ChangeExtension(fname, ".pdf");
    var tiffName = Path.ChangeExtension(fname, ".tiff");

    if (!yes2all)
    {
        Console.WriteLine($"Press ENTER to create '{docxName}' or enter an alternative file name:");
        var t = Console.ReadLine();
        if (!string.IsNullOrWhiteSpace(t))
            docxName = t;
        docxName = Path.GetFullPath(docxName);
        if (File.Exists(docxName))
        {
            Console.WriteLine($"'{docxName}' exists.\nOVERWRITE (Y/n)?");
            if (Console.ReadLine()?.ToLower() == "n")
            {
                Console.WriteLine("Aborted.");
                return;
            }
        }
    }
    if (!quiet)
        Console.WriteLine($"Generating '{docxName}'...");
    var sample = new DsWordWeb.Demos.TateChuYoko();
    GcWordDocument docx;
    docx = sample.CreateDocx();
    docx.Save(docxName);
    if (!quiet)
        Console.WriteLine($"Created '{docxName}' successfully.");

    // Export to PDF or images does not yet fully support some Word features,
    // for samples demoing those features export is disabled by default:
    var canExport = true;
    if (canExport)
    {
        if (!yes2all)
        {
            if (File.Exists(pdfName))
            {
                Console.WriteLine($"'{pdfName}' exists.\nOVERWRITE (Y/n)?");
                if (Console.ReadLine()?.ToLower() == "n")
                {
                    Console.WriteLine("Aborted.");
                    return;
                }
            }
            if (File.Exists(tiffName))
            {
                Console.WriteLine($"'{tiffName}' exists.\nOVERWRITE (Y/n)?");
                if (Console.ReadLine()?.ToLower() == "n")
                {
                    Console.WriteLine("Aborted.");
                    return;
                }
            }
        }
        var wls = typeof(DsWordWeb.Demos.TateChuYoko).GetMethod("GetWordLayoutSettings", BindingFlags.Static | BindingFlags.Public)?.Invoke(null, null) as WordLayoutSettings;
        var pdfs = typeof(DsWordWeb.Demos.TateChuYoko).GetMethod("GetPdfOutputSettings", BindingFlags.Static | BindingFlags.Public)?.Invoke(null, null) as PdfOutputSettings;
        if (!quiet)
            Console.WriteLine($"Generating '{pdfName}' and '{tiffName}'...");
        using var layout = new GcWordLayout(docx, wls);
        layout.SaveAsPdf(pdfName, null, pdfs);
        layout.SaveAsTiff(tiffName);
    }

    // Uncomment to open the generated DOCX file:
    // var psiDocx = new ProcessStartInfo() { FileName = docxName, UseShellExecute = true };
    // Process.Start(psiDocx);

    // Uncomment to open the generated PDF file:
    // var psiPdf = new ProcessStartInfo() { FileName = pdfName, UseShellExecute = true };
    // Process.Start(psiPdf);

    // Uncomment to open the generated TIFF file:
    // var psiTiff = new ProcessStartInfo() { FileName = tiffName, UseShellExecute = true };
    // Process.Start(psiTiff);
}
catch (Exception ex)
{
    Console.Error.WriteLine($"Error: {ex.Message}");
    if (ex.InnerException != null)
        Console.Error.WriteLine($"InnerException: {ex.InnerException.Message}");
}
finally
{
    if (!yes2all)
    {
        Console.WriteLine("Press any key to exit...");
        Console.ReadKey();
    }
    if (quiet)
    {
        outTw.Dispose();
        outMs.Dispose();
    }
}
